@@ -2,6 +2,7 @@ import dbus
 import os
 import re
 import six
+import tempfile
 import time
 import unittest
 
@@ -20,7 +21,7 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
             raise RuntimeError('Failed to determine cryptsetup version from: %s' % out)
         return LooseVersion(m.groups()[0])
 
-    def _create_luks(self, device, passphrase):
+    def _create_luks(self, device, passphrase, binary=False):
         raise NotImplementedError()
 
     def _remove_luks(self, device, close=True):
@@ -201,6 +202,59 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
         luks_path = self.get_property(luks_obj, '.Block', 'PreferredDevice')
         luks_path.assertEqual(self.str_to_ay(dm_path))
 
+    @unittest.skipUnless("JENKINS_HOME" in os.environ, "skipping test that modifies system configuration")
+    def test_open_crypttab_keyfile(self):
+        # this test will change /etc/crypttab, we might want to revert the changes when it finishes
+        crypttab = self.read_file('/etc/crypttab')
+        self.addCleanup(self.write_file, '/etc/crypttab', crypttab)
+
+        passwd = b'test\0test'
+        luks_name = 'myshinylittleluks'
+
+        # create key file
+        _fd, key_file = tempfile.mkstemp()
+        self.addCleanup(self.remove_file, key_file)
+
+        self.write_file(key_file, passwd, binary=True)
+
+        disk_name = os.path.basename(self.vdevs[0])
+        disk = self.get_object('/block_devices/' + disk_name)
+
+        self._create_luks(disk, passwd, binary=True)
+        self.addCleanup(self._remove_luks, disk)
+        self.udev_settle()
+
+        _ret, disk_uuid = self.run_command('lsblk -d -no UUID %s' % self.vdevs[0])
+
+        # lock the device
+        disk.Lock(self.no_options, dbus_interface=self.iface_prefix + '.Encrypted')
+
+        # add new entry to the crypttab
+        new_crypttab = crypttab + '%s UUID=%s %s\n' % (luks_name, disk_uuid, key_file)
+        self.write_file('/etc/crypttab', new_crypttab)
+
+        # give udisks time to react to change of the file
+        time.sleep(5)
+        dbus_conf = disk.GetSecretConfiguration(self.no_options, dbus_interface=self.iface_prefix + '.Block')
+        self.assertIsNotNone(dbus_conf)
+        self.assertEqual(self.ay_to_str(dbus_conf[0][1]['name']), luks_name)
+        self.assertEqual(self.ay_to_str(dbus_conf[0][1]['passphrase-path']), key_file)
+
+        # unlock the device using empty passphrase (should use the key file)
+        luks = disk.Unlock('', self.no_options,
+                           dbus_interface=self.iface_prefix + '.Encrypted')
+        self.assertIsNotNone(luks)
+
+        # unlock should use name from crypttab for the /dev/mapper device
+        dm_path = '/dev/mapper/%s' % luks_name
+        self.assertTrue(os.path.exists(dm_path))
+
+        # preferred 'device' should be /dev/mapper/name too
+        luks_obj = self.get_object(luks)
+        self.assertIsNotNone(luks_obj)
+        luks_path = self.get_property(luks_obj, '.Block', 'PreferredDevice')
+        luks_path.assertEqual(self.str_to_ay(dm_path))
+
     def test_mount(self):
         disk_name = os.path.basename(self.vdevs[0])
         disk = self.get_object('/block_devices/' + disk_name)
@@ -285,8 +339,13 @@ class UdisksEncryptedTestLUKS1(UdisksEncryptedTest):
 
     luks_version = '1'
 
-    def _create_luks(self, device, passphrase):
-        device.Format('xfs', {'encrypt.passphrase': passphrase},
+    def _create_luks(self, device, passphrase, binary=False):
+        options = dbus.Dictionary(signature='sv')
+        if binary:
+            options['encrypt.passphrase'] = self.bytes_to_ay(passphrase)
+        else:
+            options['encrypt.passphrase'] = passphrase
+        device.Format('xfs', options,
                       dbus_interface=self.iface_prefix + '.Block')
 
     def _get_metadata_size_from_dump(self, disk):
@@ -305,8 +364,15 @@ class UdisksEncryptedTestLUKS2(UdisksEncryptedTest):
 
     luks_version = '2'
 
-    def _create_luks(self, device, passphrase):
-        device.Format('xfs', {'encrypt.passphrase': passphrase, 'encrypt.type': 'luks2'},
+    def _create_luks(self, device, passphrase, binary=False):
+        options = dbus.Dictionary(signature='sv')
+        if binary:
+            options['encrypt.passphrase'] = self.bytes_to_ay(passphrase)
+            options['encrypt.type'] = 'luks2'
+        else:
+            options['encrypt.passphrase'] = passphrase
+            options['encrypt.type'] = 'luks2'
+        device.Format('xfs', options,
                       dbus_interface=self.iface_prefix + '.Block')
 
     def _get_metadata_size_from_dump(self, disk):
@@ -319,6 +385,17 @@ class UdisksEncryptedTestLUKS2(UdisksEncryptedTest):
             self.fail("Failed to get LUKS 2 offset information using 'cryptsetup luksDump %s'" % disk)
         return int(m.group(1))
 
+    def _get_key_location(self, device):
+        ret, out = self.run_command('cryptsetup status %s' % device)
+        if ret != 0:
+            self.fail('Failed to get key location from:\n%s' % out)
+
+        m = re.search(r'\s*key location:\s*(\S+)\s*', out)
+        if not m or len(m.groups()) != 1:
+            self.fail('Failed to get key location from:\n%s' % out)
+
+        return m.group(1)
+
     def setUp(self):
         cryptsetup_version = self._get_cryptsetup_version()
         if cryptsetup_version < LooseVersion('2.0.0'):
@@ -338,16 +415,18 @@ class UdisksEncryptedTestLUKS2(UdisksEncryptedTest):
         self.assertEqual(_ret, 0)
         clear_size = self.get_block_size(clear_dev)
 
-        # no passphrase for LUKS 2 = fail
-        msg = 'org.freedesktop.UDisks2.Error.Failed: Error resizing encrypted device /dev/dm-[0-9]+: Insufficient persmissions to resize device. *'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
-            device.Resize(dbus.UInt64(100*1024*1024), self.no_options,
-                          dbus_interface=self.iface_prefix + '.Encrypted')
+        # kernel keyring support and no passphrase for LUKS 2 given = fail
+        if self._get_key_location('/dev/' + clear_dev) == 'keyring':
+            msg = 'org.freedesktop.UDisks2.Error.Failed: Error resizing encrypted device /dev/dm-[0-9]+: Insufficient persmissions to resize device. *'
+            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+                device.Resize(dbus.UInt64(100*1024*1024), self.no_options,
+                              dbus_interface=self.iface_prefix + '.Encrypted')
 
         # wrong passphrase
         d = dbus.Dictionary(signature='sv')
         d['passphrase'] = 'wrongpassphrase'
-        msg = 'org.freedesktop.UDisks2.Error.Failed: Error resizing encrypted device /dev/dm-[0-9]+: Failed to activate device: Operation not permitted'
+        msg = 'org.freedesktop.UDisks2.Error.Failed: Error resizing encrypted device /dev/dm-[0-9]+: '\
+              'Failed to activate device: (Operation not permitted|Incorrect passphrase)'
         with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
             device.Resize(dbus.UInt64(100*1024*1024), d,
                           dbus_interface=self.iface_prefix + '.Encrypted')
