@@ -5,10 +5,15 @@ import dbus
 import subprocess
 import os
 import time
+import re
 import sys
 from datetime import datetime
 from systemd import journal
-from monotonic import monotonic
+
+if sys.version_info.major == 3 and sys.version_info.minor >= 3:
+    from time import monotonic
+else:
+    from monotonic import monotonic
 
 import gi
 gi.require_version('GUdev', '1.0')
@@ -37,6 +42,50 @@ def get_call_long(call):
     return call_long
 
 
+def get_version_from_pretty_name(pretty_name):
+    """ Try to get distro and version from 'OperatingSystemPrettyName'
+        hostname property.
+
+        It should look like this:
+         - "Debian GNU/Linux 9 (stretch)"
+         - "Fedora 27 (Workstation Edition)"
+         - "CentOS Linux 7 (Core)"
+
+        So just return first word as distro and first number as version.
+    """
+    distro = pretty_name.split()[0].lower()
+    match = re.search(r"\d+", pretty_name)
+    if match is not None:
+        version = match.group(0)
+    else:
+        raise RuntimeError("Cannot get distro name and version from '%s'" % pretty_name)
+
+    return (distro, version)
+
+
+def get_version():
+    """ Try to get distro and version
+    """
+
+    bus = dbus.SystemBus()
+
+    # get information about the distribution from systemd (hostname1)
+    sys_info = bus.get_object("org.freedesktop.hostname1", "/org/freedesktop/hostname1")
+    cpe = str(sys_info.Get("org.freedesktop.hostname1", "OperatingSystemCPEName", dbus_interface=dbus.PROPERTIES_IFACE))
+
+    if cpe:
+        # 2nd to 4th fields from e.g. "cpe:/o:fedoraproject:fedora:25" or "cpe:/o:redhat:enterprise_linux:7.3:GA:server"
+        _project, distro, version = tuple(cpe.split(":")[2:5])
+    else:
+        pretty_name = str(sys_info.Get("org.freedesktop.hostname1", "OperatingSystemPrettyName", dbus_interface=dbus.PROPERTIES_IFACE))
+        distro, version = get_version_from_pretty_name(pretty_name)
+
+    # we want just the major version, so remove all decimal places (if any)
+    version = str(int(float(version)))
+
+    return (distro, version)
+
+
 def skip_on(skip_on_distros, skip_on_version="", reason=""):
     """A function returning a decorator to skip some test on a given distribution-version combination
 
@@ -49,14 +98,7 @@ def skip_on(skip_on_distros, skip_on_version="", reason=""):
     if isinstance(skip_on_distros, str):
         skip_on_distros = (skip_on_distros,)
 
-    bus = dbus.SystemBus()
-
-    # get information about the distribution from systemd (hostname1)
-    sys_info = bus.get_object("org.freedesktop.hostname1", "/org/freedesktop/hostname1")
-    cpe = str(sys_info.Get("org.freedesktop.hostname1", "OperatingSystemCPEName", dbus_interface=dbus.PROPERTIES_IFACE))
-
-    # 2nd to 4th fields from e.g. "cpe:/o:fedoraproject:fedora:25" or "cpe:/o:redhat:enterprise_linux:7.3:GA:server"
-    project, distro, version = tuple(cpe.split(":")[2:5])
+    distro, version = get_version()
 
     def decorator(func):
         if distro in skip_on_distros and (not skip_on_version or skip_on_version == version):
@@ -80,12 +122,12 @@ def unstable_test(test):
         except unittest.SkipTest:
             # make sure skipped tests are just skipped as usual
             raise
-        except:
+        except Exception as e:
             # and swallow everything else, just report a failure of an unstable
             # test, unless told otherwise
             if "UNSTABLE_TESTS_FATAL" in os.environ:
                 raise
-            print("unstable-fail...", end="", file=sys.stderr)
+            print("unstable-fail: Ignoring exception '%s'\n" % e, end="", file=sys.stderr)
 
     return decorated_test
 
@@ -158,7 +200,7 @@ class DBusProperty(object):
         ret = self._check(timeout, check_fn)
 
         if not ret:
-            raise AssertionError('%s is not grater than %s' % (self._value, value))
+            raise AssertionError('%s is not greater than %s' % (self._value, value))
 
     def assertLess(self, value, timeout=TIMEOUT):
         check_fn = lambda x: x < value
@@ -237,12 +279,7 @@ class UdisksTestCase(unittest.TestCase):
         self.path_prefix = '/org/freedesktop/UDisks2'
         self.bus = dbus.SystemBus()
 
-        # get information about the distribution from systemd (hostname1)
-        sys_info = self.bus.get_object("org.freedesktop.hostname1", "/org/freedesktop/hostname1")
-        cpe = str(sys_info.Get("org.freedesktop.hostname1", "OperatingSystemCPEName", dbus_interface=dbus.PROPERTIES_IFACE))
-
-        # 2nd to 4th fields from e.g. "cpe:/o:fedoraproject:fedora:25" or "cpe:/o:redhat:enterprise_linux:7.3:GA:server"
-        self.distro = tuple(cpe.split(":")[2:5])
+        self.distro = get_version()
 
         self._orig_call_async = self.bus.call_async
         self._orig_call_blocking = self.bus.call_blocking
@@ -350,11 +387,19 @@ class UdisksTestCase(unittest.TestCase):
 
 
     @classmethod
-    def write_file(self, filename, content, ignore_nonexistent=False):
+    def write_file(self, filename, content, ignore_nonexistent=False, binary=False):
         try:
-            with open(filename, 'w') as f:
+            with open(filename, 'wb' if binary else 'w') as f:
                 f.write(content)
-        except FileNotFoundError as e:
+        except OSError as e:
+            if not ignore_nonexistent:
+                raise e
+
+    @classmethod
+    def remove_file(self, filename, ignore_nonexistent=False):
+        try:
+            os.remove(filename)
+        except OSError as e:
             if not ignore_nonexistent:
                 raise e
 
@@ -385,14 +430,22 @@ class UdisksTestCase(unittest.TestCase):
         return ''.join(chr(x) for x in ay[:-1])
 
     @classmethod
-    def str_to_ay(self, string):
+    def str_to_ay(self, string, terminate=True):
         """Convert a string to a bytearray (terminated with '\0')"""
 
-        string += '\0'
+        if terminate:
+            string += '\0'
 
         return dbus.Array([dbus.Byte(ord(c)) for c in string],
                           signature=dbus.Signature('y'), variant_level=1)
 
+    @classmethod
+    def bytes_to_ay(self, bytes):
+        """Convert Python bytes to a DBus bytearray"""
+
+        return dbus.Array([dbus.Byte(b) for b in bytes],
+                          signature=dbus.Signature('y'), variant_level=1)
+
     @classmethod
     def set_udev_property(self, device, prop, value):
         udev = GUdev.Client()
