@@ -94,6 +94,7 @@ struct _UDisksDaemon
   gboolean disable_modules;
   gboolean force_load_modules;
   gboolean uninstalled;
+  gboolean enable_tcrypt;
 };
 
 struct _UDisksDaemonClass
@@ -114,6 +115,7 @@ enum
   PROP_DISABLE_MODULES,
   PROP_FORCE_LOAD_MODULES,
   PROP_UNINSTALLED,
+  PROP_ENABLE_TCRYPT,
 };
 
 G_DEFINE_TYPE (UDisksDaemon, udisks_daemon, G_TYPE_OBJECT);
@@ -198,6 +200,10 @@ udisks_daemon_get_property (GObject    *object,
       g_value_set_boolean (value, udisks_daemon_get_uninstalled (daemon));
       break;
 
+    case PROP_ENABLE_TCRYPT:
+      g_value_set_boolean (value, udisks_daemon_get_enable_tcrypt (daemon));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -231,6 +237,10 @@ udisks_daemon_set_property (GObject      *object,
       daemon->uninstalled = g_value_get_boolean (value);
       break;
 
+    case PROP_ENABLE_TCRYPT:
+      daemon->enable_tcrypt = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -481,6 +491,21 @@ udisks_daemon_class_init (UDisksDaemonClass *klass)
                                                          G_PARAM_READABLE |
                                                          G_PARAM_WRITABLE |
                                                          G_PARAM_CONSTRUCT_ONLY));
+
+  /**
+   * UDisksDaemon:enable-tcrypt:
+   *
+   * Whether devices should be tested for being TCRYPT encrypted.
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_ENABLE_TCRYPT,
+                                   g_param_spec_boolean ("enable-tcrypt",
+                                                         "Enable TCRYPT",
+                                                         "Whether devices should be tested for being TCRYPT encrypted",
+                                                         FALSE,
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_WRITABLE |
+                                                         G_PARAM_CONSTRUCT_ONLY));
 }
 
 /**
@@ -489,6 +514,7 @@ udisks_daemon_class_init (UDisksDaemonClass *klass)
  * @disable_modules: Indicates whether modules should never be activated.
  * @force_load_modules: Activate modules on startup (for debugging purposes).
  * @uninstalled: Loads modules from the build directory (for debugging purposes).
+ * @enable_tcrypt: Checks whether devices could be TCRYPT encrypted.
  *
  * Create a new daemon object for exporting objects on @connection.
  *
@@ -498,7 +524,8 @@ UDisksDaemon *
 udisks_daemon_new (GDBusConnection *connection,
                    gboolean         disable_modules,
                    gboolean         force_load_modules,
-                   gboolean         uninstalled)
+                   gboolean         uninstalled,
+                   gboolean         enable_tcrypt)
 {
   g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), NULL);
   return UDISKS_DAEMON (g_object_new (UDISKS_TYPE_DAEMON,
@@ -506,6 +533,7 @@ udisks_daemon_new (GDBusConnection *connection,
                                       "disable-modules", disable_modules,
                                       "force-load-modules", force_load_modules,
                                       "uninstalled", uninstalled,
+                                      "enable-tcrypt", enable_tcrypt,
                                       NULL));
 }
 
@@ -650,13 +678,28 @@ udisks_daemon_get_state (UDisksDaemon *daemon)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+typedef struct
+{
+  UDisksDaemon *daemon;
+  UDisksInhibitCookie *inhibit_cookie;
+} JobData;
+
+static void
+free_job_data (JobData *job_data)
+{
+  if (job_data->daemon != NULL)
+    g_object_unref (job_data->daemon);
+  g_free (job_data);
+}
+
 static void
 on_job_completed (UDisksJob    *job,
                   gboolean      success,
                   const gchar  *message,
                   gpointer      user_data)
 {
-  UDisksDaemon *daemon = UDISKS_DAEMON (user_data);
+  JobData *job_data = (JobData *) user_data;
+  UDisksDaemon *daemon = UDISKS_DAEMON (job_data->daemon);
   UDisksObjectSkeleton *object;
 
   object = UDISKS_OBJECT_SKELETON (g_dbus_interface_get_object (G_DBUS_INTERFACE (job)));
@@ -670,11 +713,13 @@ on_job_completed (UDisksJob    *job,
   /* free the allocated job object */
   g_object_unref (job);
 
+  /* unregister inhibitor from systemd logind */
+  udisks_daemon_util_uninhibit_system_sync (job_data->inhibit_cookie);
   /* returns the reference we took when connecting to the
    * UDisksJob::completed signal in udisks_daemon_launch_{spawned,threaded}_job()
    * below
    */
-  g_object_unref (daemon);
+  free_job_data (job_data);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -692,6 +737,15 @@ common_job (UDisksDaemon    *daemon,
 {
   gchar *job_object_path;
   UDisksObjectSkeleton *job_object;
+  JobData *job_data;
+  gchar *operation_description;
+
+  job_data = g_new0 (JobData, 1);
+  job_data->daemon = g_object_ref (daemon);
+  /* register inhibitor to systemd logind while job is running */
+  operation_description = udisks_client_get_job_description_from_operation (job_operation);
+  job_data->inhibit_cookie = udisks_daemon_util_inhibit_system_sync (operation_description);
+  g_free (operation_description);
 
   if (object != NULL)
     udisks_base_job_add_object (UDISKS_BASE_JOB (job), object);
@@ -709,7 +763,7 @@ common_job (UDisksDaemon    *daemon,
   g_signal_connect_after (job,
                           "completed",
                           G_CALLBACK (on_job_completed),
-                          g_object_ref (daemon));
+                          job_data);
 
   return UDISKS_BASE_JOB (job);
 }
@@ -1132,37 +1186,41 @@ udisks_daemon_launch_spawned_job_gstring_sync (UDisksDaemon    *daemon,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-typedef struct
-{
-  GMainContext *context;
-  GMainLoop    *loop;
-  gboolean      result;
-  GError       *error;
-} ThreadedJobSyncData;
-
-static gboolean
-threaded_job_sync_on_threaded_job_completed (UDisksThreadedJob *job,
-                                             gboolean           result,
-                                             GError            *error,
-                                             gpointer           user_data)
-{
-  ThreadedJobSyncData *data = user_data;
-  data->result = result;
-  if (error)
-    data->error = g_error_copy (error);
-  return FALSE; /* let other handlers run */
-}
+static __thread UDisksJob *thread_job = NULL;
 
 static void
-threaded_job_sync_on_completed (UDisksJob    *job,
-                                gboolean      success,
-                                const gchar  *message,
-                                gpointer      user_data)
+bd_thread_progress_callback (guint64            task_id,
+                             BDUtilsProgStatus  status,
+                             guint8             completion,
+                             gchar             *msg)
 {
-  ThreadedJobSyncData *data = user_data;
-  g_main_loop_quit (data->loop);
+  if (thread_job != NULL && msg == NULL)
+    {
+      if (!udisks_job_get_progress_valid (UDISKS_JOB (thread_job)))
+        {
+          udisks_job_set_progress_valid (UDISKS_JOB (thread_job), TRUE);
+        }
+
+      udisks_job_set_progress (UDISKS_JOB (thread_job), completion / 100.0);
+    }
 }
 
+void
+udisks_bd_thread_set_progress_for_job (UDisksJob *job)
+{
+  thread_job = job;
+  bd_utils_init_prog_reporting_thread (bd_thread_progress_callback, NULL);
+}
+
+void
+udisks_bd_thread_disable_progress (void)
+{
+  thread_job = NULL;
+  bd_utils_init_prog_reporting_thread (NULL, NULL);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 /**
  * udisks_daemon_launch_threaded_job_sync:
  * @daemon: A #UDisksDaemon.
@@ -1192,17 +1250,11 @@ udisks_daemon_launch_threaded_job_sync (UDisksDaemon          *daemon,
                                         GError               **error)
 {
   UDisksBaseJob *job;
-  ThreadedJobSyncData data;
+  gboolean job_result;
 
   g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), FALSE);
   g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
 
-  data.context = g_main_context_new ();
-  g_main_context_push_thread_default (data.context);
-  data.loop = g_main_loop_new (data.context, FALSE);
-  data.error = NULL;
-  data.result = FALSE;
-
   job = udisks_daemon_launch_threaded_job (daemon,
                                            object,
                                            job_operation,
@@ -1211,27 +1263,14 @@ udisks_daemon_launch_threaded_job_sync (UDisksDaemon          *daemon,
                                            user_data,
                                            user_data_free_func,
                                            cancellable);
-  g_signal_connect (job,
-                    "threaded-job-completed",
-                    G_CALLBACK (threaded_job_sync_on_threaded_job_completed),
-                    &data);
-  g_signal_connect_after (job,
-                          "completed",
-                          G_CALLBACK (threaded_job_sync_on_completed),
-                          &data);
 
-  udisks_threaded_job_start (UDISKS_THREADED_JOB (job));
-  g_main_loop_run (data.loop);
-
-  g_main_loop_unref (data.loop);
-  g_main_context_pop_thread_default (data.context);
-  g_main_context_unref (data.context);
-
-  if (data.error)
-    g_propagate_error (error, data.error);
+  /* TODO: There might not be much difference between calling the job_func() right away
+   *       instead of having it enclosed in a GTask since we're blocking anyway.
+   *       Deeper investigation of differences required. */
+  job_result = udisks_threaded_job_run_sync (UDISKS_THREADED_JOB (job), error);
 
   /* note: the job object is freed in the ::completed handler */
-  return data.result;
+  return job_result;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1686,6 +1725,23 @@ udisks_daemon_get_uninstalled (UDisksDaemon *daemon)
   return daemon->uninstalled;
 }
 
+/**
+ * udisks_daemon_get_enable_tcrypt:
+ * @daemon: A #UDisksDaemon.
+ *
+ * Gets @daemon setting whether devices should be tested for being TCRYPT
+ * encrypted.
+ *
+ * Returns: %TRUE if "/etc/udisks2/tcrypt.conf" was an existing file during
+ * daemon startup.
+ */
+gboolean
+udisks_daemon_get_enable_tcrypt (UDisksDaemon *daemon)
+{
+  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), FALSE);
+  return daemon->enable_tcrypt;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 gchar *
