@@ -293,6 +293,41 @@ udisks_safe_append_to_object_path (GString      *str,
     }
 }
 
+/**
+ * udisks_g_object_ref_foreach:
+ * @object: A #GObject to ref.
+ * @unused: Unused parameter.
+ *
+ * This is a helper function for g_list_foreach. It expects a function
+ * that takes two parameters but the standard g_object_ref takes just one
+ * and using it makes gcc sad. So this function just calls g_object_ref
+ * and throws away the second parameter.
+ */
+void
+udisks_g_object_ref_foreach (gpointer object, gpointer unused)
+{
+  g_return_if_fail (G_IS_OBJECT (object));
+  g_object_ref (G_OBJECT (object));
+  return;
+}
+
+/**
+ * udisks_g_object_ref_copy:
+ * @object: A #GObject to ref.
+ * @unused: Unused parameter.
+ *
+ * This is a helper function for g_list_copy_deep. It expects copy function
+ * that takes two parameters but the standard g_object_ref takes just one
+ * and using it makes gcc sad. So this function just calls g_object_ref
+ * and throws away the second parameter.
+ */
+void *
+udisks_g_object_ref_copy (gconstpointer object, gpointer unused)
+{
+  g_return_val_if_fail (G_IS_OBJECT (object), NULL);
+  return g_object_ref (G_OBJECT (object));
+}
+
 /**
  * udisks_daemon_util_block_get_size:
  * @device: A #GUdevDevice for a top-level block device.
@@ -519,9 +554,9 @@ udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
     {
       UDisksBlock *crypto_block;
       crypto_block = udisks_object_peek_block (crypto_object);
-      if (udisks_state_find_unlocked_luks (state,
-                                           udisks_block_get_device_number (crypto_block),
-                                           &setup_by_user))
+      if (udisks_state_find_unlocked_crypto_dev (state,
+                                                 udisks_block_get_device_number (crypto_block),
+                                                 &setup_by_user))
         {
           if (setup_by_user == user)
             {
@@ -597,8 +632,6 @@ check_authorization_no_polkit (UDisksDaemon            *daemon,
                                                invocation,
                                                NULL,         /* GCancellable* */
                                                &caller_uid,
-                                               NULL,         /* gid_t *out_gid */
-                                               NULL,         /* gchar **out_user_name */
                                                &sub_error))
     {
       g_set_error (error,
@@ -967,6 +1000,56 @@ out:
   return ret;
 }
 
+/**
+ * udisks_daemon_util_get_user_info:
+ * @out_gid: (out) (allow-none): Return location for resolved gid or %NULL.
+ * @out_user_name: (out) (allow-none): Return location for resolved user name or %NULL.
+ * @error: Return location for error.
+ *
+ * Gets the UNIX group and user name for a user id.
+ *
+ * Returns: %TRUE if the user information was obtained, %FALSE otherwise
+ */
+gboolean
+udisks_daemon_util_get_user_info (const uid_t uid,
+                                  gid_t *out_gid,
+                                  gchar **out_user_name,
+                                  GError     **error)
+{
+  struct passwd pwstruct;
+  gchar pwbuf[8192];
+  struct passwd *pw = NULL;
+  int rc;
+
+  rc = getpwuid_r (uid, &pwstruct, pwbuf, sizeof pwbuf, &pw);
+  if (rc == 0 && pw == NULL)
+    {
+      g_set_error (error,
+                   UDISKS_ERROR,
+                   UDISKS_ERROR_FAILED,
+                   "User with uid %d does not exist", (gint) uid);
+      goto out;
+    }
+  else if (pw == NULL)
+    {
+      g_set_error (error,
+                   UDISKS_ERROR,
+                   UDISKS_ERROR_FAILED,
+                   "Error looking up passwd struct for uid %d: %m", (gint) uid);
+      goto out;
+    }
+
+  if (out_gid != NULL)
+    *out_gid = pw->pw_gid;
+
+  if (out_user_name != NULL)
+      *out_user_name = g_strdup (pwstruct.pw_name);
+
+  return TRUE;
+
+out:
+  return FALSE;
+}
 
 /**
  * udisks_daemon_util_get_caller_uid_sync:
@@ -974,12 +1057,9 @@ out:
  * @invocation: A #GDBusMethodInvocation.
  * @cancellable: (allow-none): A #GCancellable or %NULL.
  * @out_uid: (out): Return location for resolved uid or %NULL.
- * @out_gid: (out) (allow-none): Return location for resolved gid or %NULL.
- * @out_user_name: (out) (allow-none): Return location for resolved user name or %NULL.
  * @error: Return location for error.
  *
- * Gets the UNIX user id (and possibly group id and user name) of the
- * peer represented by @invocation.
+ * Gets the UNIX user id of the peer represented by @invocation.
  *
  * Returns: %TRUE if the user id (and possibly group id) was obtained, %FALSE otherwise
  */
@@ -988,8 +1068,6 @@ udisks_daemon_util_get_caller_uid_sync (UDisksDaemon            *daemon,
                                         GDBusMethodInvocation   *invocation,
                                         GCancellable            *cancellable,
                                         uid_t                   *out_uid,
-                                        gid_t                   *out_gid,
-                                        gchar                  **out_user_name,
                                         GError                 **error)
 {
   gboolean ret;
@@ -1009,36 +1087,6 @@ udisks_daemon_util_get_caller_uid_sync (UDisksDaemon            *daemon,
   if (out_uid != NULL)
     *out_uid = uid;
 
-  if (out_gid != NULL || out_user_name != NULL)
-    {
-      struct passwd pwstruct;
-      gchar pwbuf[8192];
-      struct passwd *pw = NULL;
-      int rc;
-
-      rc = getpwuid_r (uid, &pwstruct, pwbuf, sizeof pwbuf, &pw);
-      if (rc == 0 && pw == NULL)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "User with uid %d does not exist", (gint) uid);
-          goto out;
-        }
-      else if (pw == NULL)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "Error looking up passwd struct for uid %d: %m", (gint) uid);
-          goto out;
-        }
-      if (out_gid != NULL)
-        *out_gid = pw->pw_gid;
-      if (out_user_name != NULL)
-        *out_user_name = g_strdup (pwstruct.pw_name);
-    }
-
   ret = TRUE;
 
  out:
@@ -1552,7 +1600,7 @@ udisks_daemon_util_uninhibit_system_sync (UDisksInhibitCookie *cookie)
       g_assert (cookie->magic == 0xdeadbeef);
       if (close (cookie->fd) != 0)
         {
-          udisks_critical ("Error closing inhbit-fd: %m");
+          udisks_critical ("Error closing inhibit-fd: %m");
         }
       g_free (cookie);
     }
