@@ -338,7 +338,7 @@ static const gchar *vfat_allow_gid_self[] = { "gid", NULL };
 /* ---------------------- ntfs -------------------- */
 /* this is assuming that ntfs-3g is used */
 
-static const gchar *ntfs_defaults[] = { "uid=", "gid=", NULL };
+static const gchar *ntfs_defaults[] = { "uid=", "gid=", "windows_names", NULL };
 static const gchar *ntfs_allow[] = { "umask", "dmask", "fmask", "locale", "norecover", "ignore_case", "windows_names", "compression", "nocompression", "big_writes", NULL };
 static const gchar *ntfs_allow_uid_self[] = { "uid", NULL };
 static const gchar *ntfs_allow_gid_self[] = { "gid", NULL };
@@ -1358,8 +1358,6 @@ handle_mount (UDisksFilesystem      *filesystem,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               &caller_user_name,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1367,6 +1365,13 @@ handle_mount (UDisksFilesystem      *filesystem,
       goto out;
     }
 
+  if (!udisks_daemon_util_get_user_info (caller_uid, &caller_gid, &caller_user_name, &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_clear_error (&error);
+      goto out;
+    }
+
   if (system_managed)
     {
       gboolean mount_fstab_as_root = FALSE;
@@ -1731,7 +1736,7 @@ handle_unmount (UDisksFilesystem      *filesystem,
     }
 
   error = NULL;
-  if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL, &caller_uid, NULL, NULL, &error))
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL, &caller_uid, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
       g_clear_error (&error);
@@ -1917,7 +1922,6 @@ handle_set_label (UDisksFilesystem      *filesystem,
   const gchar *message;
   gchar *real_label = NULL;
   uid_t caller_uid;
-  gid_t caller_gid;
   gchar *command;
   gint status = 0;
   gchar *out_message = NULL;
@@ -1945,8 +1949,6 @@ handle_set_label (UDisksFilesystem      *filesystem,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -2121,7 +2123,6 @@ handle_resize (UDisksFilesystem      *filesystem,
   const gchar *action_id = NULL;
   const gchar *message = NULL;
   uid_t caller_uid;
-  gid_t caller_gid;
   GError *error = NULL;
   UDisksBaseJob *job = NULL;
   gchar *required_utility = NULL;
@@ -2143,8 +2144,6 @@ handle_resize (UDisksFilesystem      *filesystem,
                                                 invocation,
                                                 NULL /* GCancellable */,
                                                 &caller_uid,
-                                                &caller_gid,
-                                                NULL,
                                                 &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -2261,6 +2260,10 @@ handle_resize (UDisksFilesystem      *filesystem,
       goto out;
     }
 
+  /* At least resize2fs might need another uevent after it is done.
+   */
+  udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (object));
+
   udisks_filesystem_complete_resize (filesystem, invocation);
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
@@ -2289,7 +2292,6 @@ handle_repair (UDisksFilesystem      *filesystem,
   const gchar *action_id = NULL;
   const gchar *message = NULL;
   uid_t caller_uid;
-  gid_t caller_gid;
   GError *error = NULL;
   gboolean ret = FALSE;
   UDisksBaseJob *job = NULL;
@@ -2312,8 +2314,6 @@ handle_repair (UDisksFilesystem      *filesystem,
                                                 invocation,
                                                 NULL /* GCancellable */,
                                                 &caller_uid,
-                                                &caller_gid,
-                                                NULL,
                                                 &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -2449,7 +2449,6 @@ handle_check (UDisksFilesystem      *filesystem,
   const gchar *action_id = NULL;
   const gchar *message = NULL;
   uid_t caller_uid;
-  gid_t caller_gid;
   GError *error = NULL;
   gboolean ret = FALSE;
   UDisksBaseJob *job = NULL;
@@ -2472,8 +2471,6 @@ handle_check (UDisksFilesystem      *filesystem,
                                                 invocation,
                                                 NULL /* GCancellable */,
                                                 &caller_uid,
-                                                &caller_gid,
-                                                NULL,
                                                 &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -2631,14 +2628,19 @@ handle_take_ownership (UDisksFilesystem      *filesystem,
                                                 invocation,
                                                 NULL /* GCancellable */,
                                                 &caller_uid,
-                                                &caller_gid,
-                                                NULL,
                                                 &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
       goto out;
     }
 
+  if (!udisks_daemon_util_get_user_info (caller_uid, &caller_gid, NULL /* user name */, &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_clear_error (&error);
+      goto out;
+    }
+
   probed_fs_usage = udisks_block_get_id_usage (block);
   if (g_strcmp0 (probed_fs_usage, "filesystem") != 0)
     {
