@@ -365,44 +365,41 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
         }
     }
 
-  if (sync_action)
+  if (sync_action == NULL || g_strcmp0 (sync_action, "idle") == 0)
     {
-      if (g_strcmp0 (sync_action, "idle") != 0)
+      if (udisks_linux_mdraid_object_has_sync_job (object))
         {
-          if (! udisks_linux_mdraid_object_has_sync_job (object))
-            {
-              /* Launch a job */
-              job = udisks_daemon_launch_simple_job (daemon,
-                                                     UDISKS_OBJECT (object),
-                                                     sync_action_to_job_id (sync_action),
-                                                     0,
-                                                     NULL /* cancellable */);
+          /* Complete the job */
+          udisks_linux_mdraid_object_complete_sync_job (object,
+                                                        TRUE,
+                                                        "Finished");
+        }
+    }
+  else
+    {
+      if (! udisks_linux_mdraid_object_has_sync_job (object))
+        {
+          /* Launch a job */
+          job = udisks_daemon_launch_simple_job (daemon,
+                                                 UDISKS_OBJECT (object),
+                                                 sync_action_to_job_id (sync_action),
+                                                 0,
+                                                 NULL /* cancellable */);
 
-              /* Mark the job as not cancellable. It simply has to finish... */
-              udisks_job_set_cancelable (UDISKS_JOB (job), FALSE);
-              udisks_linux_mdraid_object_set_sync_job (object, job);
-            }
-          else
-            job = udisks_linux_mdraid_object_get_sync_job (object);
-
-          /* Update the job's interface */
-          udisks_job_set_progress (UDISKS_JOB (job), sync_completed_val);
-          udisks_job_set_progress_valid (UDISKS_JOB (job), TRUE);
-          udisks_job_set_rate (UDISKS_JOB (job), sync_rate);
-
-          udisks_job_set_expected_end_time (UDISKS_JOB (job),
-                                            g_get_real_time () + sync_remaining_time);
+          /* Mark the job as not cancellable. It simply has to finish... */
+          udisks_job_set_cancelable (UDISKS_JOB (job), FALSE);
+          udisks_linux_mdraid_object_set_sync_job (object, job);
         }
       else
-        {
-          if (udisks_linux_mdraid_object_has_sync_job (object))
-            {
-              /* Complete the job */
-              udisks_linux_mdraid_object_complete_sync_job (object,
-                                                            TRUE,
-                                                            "Finished");
-            }
-        }
+        job = udisks_linux_mdraid_object_get_sync_job (object);
+
+      /* Update the job's interface */
+      udisks_job_set_progress (UDISKS_JOB (job), sync_completed_val);
+      udisks_job_set_progress_valid (UDISKS_JOB (job), TRUE);
+      udisks_job_set_rate (UDISKS_JOB (job), sync_rate);
+
+      udisks_job_set_expected_end_time (UDISKS_JOB (job),
+                                        g_get_real_time () + sync_remaining_time);
     }
   udisks_mdraid_set_sync_completed (iface, sync_completed_val);
   udisks_mdraid_set_sync_rate (iface, sync_rate);
@@ -591,7 +588,6 @@ handle_start (UDisksMDRaid           *_mdraid,
   const gchar *action_id;
   const gchar *message;
   uid_t caller_uid;
-  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   GList *member_devices = NULL;
   gchar *raid_device_file = NULL;
@@ -621,8 +617,6 @@ handle_start (UDisksMDRaid           *_mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -763,7 +757,6 @@ udisks_linux_mdraid_stop (UDisksMDRaid           *_mdraid,
   UDisksLinuxMDRaidObject *object;
   uid_t started_by_uid;
   uid_t caller_uid;
-  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   UDisksBaseJob *job = NULL;
   const gchar *device_file = NULL;
@@ -783,8 +776,6 @@ udisks_linux_mdraid_stop (UDisksMDRaid           *_mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                error))
     {
       ret = FALSE;
@@ -961,7 +952,6 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
   const gchar *message;
   uid_t started_by_uid;
   uid_t caller_uid;
-  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   const gchar *device_file = NULL;
   const gchar *member_device_file = NULL;
@@ -990,8 +980,6 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1127,7 +1115,6 @@ handle_add_device (UDisksMDRaid           *_mdraid,
   const gchar *message;
   uid_t started_by_uid;
   uid_t caller_uid;
-  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   const gchar *device_file = NULL;
   const gchar *new_member_device_file = NULL;
@@ -1151,8 +1138,6 @@ handle_add_device (UDisksMDRaid           *_mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1261,7 +1246,6 @@ handle_set_bitmap_location (UDisksMDRaid           *_mdraid,
   const gchar *message;
   uid_t started_by_uid;
   uid_t caller_uid;
-  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   const gchar *device_file = NULL;
   GError *error = NULL;
@@ -1282,8 +1266,6 @@ handle_set_bitmap_location (UDisksMDRaid           *_mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1381,7 +1363,6 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
   const gchar *message;
   uid_t started_by_uid;
   uid_t caller_uid;
-  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   GError *error = NULL;
   const gchar *device_file = NULL;
@@ -1402,8 +1383,6 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1477,6 +1456,7 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
       goto out;
     }
 
+  udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
   udisks_mdraid_complete_request_sync_action (_mdraid, invocation);
 
  out:
@@ -1496,7 +1476,6 @@ udisks_linux_mdraid_delete (UDisksMDRaid           *mdraid,
   UDisksLinuxMDRaidObject *object;
   UDisksDaemon *daemon;
   uid_t caller_uid;
-  gid_t caller_gid;
   const gchar *message;
   const gchar *action_id;
   gboolean teardown_flag = FALSE;
@@ -1523,8 +1502,6 @@ udisks_linux_mdraid_delete (UDisksMDRaid           *mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                error))
     {
       ret = FALSE;
