@@ -190,8 +190,7 @@ udisks_linux_partition_table_get_partitions (UDisksDaemon         *daemon,
     }
   ret = g_list_reverse (ret);
  out:
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   return ret;
 }
 
@@ -267,9 +266,8 @@ udisks_linux_partition_table_handle_create_partition (UDisksPartitionTable   *ta
   BDPartSpec *part_spec = NULL;
   BDPartSpec *overlapping_part = NULL;
   BDPartTypeReq part_type = 0;
-  const gchar *table_type;
+  gchar *table_type = NULL;
   uid_t caller_uid;
-  gid_t caller_gid;
   GError *error = NULL;
   UDisksBaseJob *job = NULL;
   gchar *partition_type = NULL;
@@ -298,8 +296,6 @@ udisks_linux_partition_table_handle_create_partition (UDisksPartitionTable   *ta
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
